@@ -7,31 +7,31 @@ from tensorpac.methods import (get_pac_fcn, pacstr, compute_surrogates,
                                erpac, ergcpac, _ergcpac_perm, preferred_phase,
                                normalize)
 from tensorpac.gcmi import copnorm
-from tensorpac.utils import pac_vec
 from tensorpac.visu import _PacVisual, _PacPlt, _PolarPlt
 from tensorpac.io import set_log_level
-from tensorpac.config import MNE_EPOCHS_TYPE
+from tensorpac.config import CONFIG
 
 logger = logging.getLogger('tensorpac')
 
 
-
 class _PacObj(object):
     """Main class for relative PAC objects."""
 
     def __init__(self, f_pha=[2, 4], f_amp=[60, 200], dcomplex='hilbert',
-                 filt='fir1', cycle=(3, 6), filtorder=3, width=7):
+                 cycle=(3, 6), width=7):
         # Frequency checking :
+        from tensorpac.utils import pac_vec
         self._f_pha, self._f_amp = pac_vec(f_pha, f_amp)
         self._xvec, self._yvec = self.f_pha.mean(1), self.f_amp.mean(1)
         # Check spectral properties :
-        self._speccheck(filt, dcomplex, filtorder, cycle, width)
+        self._speccheck(dcomplex, cycle, width)
 
     def __str__(self):
         """String representation."""
         return self.method
 
-    def filter(self, sf, x, ftype='phase', keepfilt=False, n_jobs=-1):
+    def filter(self, sf, x, ftype='phase', keepfilt=False, edges=None,
+               n_jobs=-1):
         """Filt the data in the specified frequency bands.
 
         Parameters
@@ -49,6 +49,8 @@ class _PacObj(object):
         keepfilt : bool | False
             Specify if you only want the filtered data (True). This parameter
             is only available with dcomplex='hilbert' and not wavelet.
+        edges : int | None
+            Number of samples to discard to avoid edge effects due to filtering
 
         Returns
         -------
@@ -67,39 +69,35 @@ class _PacObj(object):
                              "'hilbert'.")
         assert ftype in ['phase', 'amplitude'], ("ftype must either be 'phase'"
                                                  " or 'amplitude.'")
-        if not isinstance(x, np.ndarray) and type(x) in MNE_EPOCHS_TYPE:
+        mne_epochs_type = CONFIG['MNE_EPOCHS_TYPE']
+        if not isinstance(x, np.ndarray) and type(x) in mne_epochs_type:
             x = x.get_data()
             sf = x.info['sfreq']
         if x.ndim == 1:
             x = x[np.newaxis, :]
         assert x.ndim == 2, ("x should be a 2d array like (n_epochs, n_times)")
+        # check edges
+        if not isinstance(edges, int):
+            edges = slice(None)
+        else:
+            logger.debug(f"    Edges {edges} time samples ignored")
+            edges = slice(edges, -edges)
+        self._edges = edges
 
         # ---------------------------------------------------------------------
         # Switch between phase or amplitude :
         if ftype is 'phase':
             tosend = 'pha' if not keepfilt else None
-            logger.info(f"    Extract {len(self.f_pha)} phases")
             xfilt = spectral(x, sf, self.f_pha, tosend, self._dcomplex,
-                             self._filt, self._filtorder, self._cycle[0],
-                             self._width, n_jobs)
+                             self._cycle[0], self._width, n_jobs)
         elif ftype is 'amplitude':
             tosend = 'amp' if not keepfilt else None
-            logger.info(f"    Extract {len(self.f_amp)} amplitudes")
             xfilt = spectral(x, sf, self.f_amp, tosend, self._dcomplex,
-                             self._filt, self._filtorder, self._cycle[1],
-                             self._width, n_jobs)
-        return xfilt
+                             self._cycle[1], self._width, n_jobs)
+        return xfilt[..., edges]
 
-    def _speccheck(self, filt=None, dcomplex=None, filtorder=None, cycle=None,
-                   width=None):
+    def _speccheck(self, dcomplex=None, cycle=None, width=None):
         """Check spectral parameters."""
-        # Check the filter name :
-        if filt is not None:
-            if filt not in ['fir1', 'butter', 'bessel', 'mne']:
-                raise ValueError("filt must either be 'fir1', 'butter' or "
-                                 "'bessel'")
-            else:
-                self._filt = filt
         # Check cycle :
         if cycle is not None:
             cycle = np.asarray(cycle)
@@ -114,9 +112,6 @@ class _PacObj(object):
                                  "'wavelet'.")
             else:
                 self._dcomplex = dcomplex
-        # Convert filtorder :
-        if filtorder is not None:
-            self._filtorder = int(filtorder)
         # Convert Morlet's width :
         if width is not None:
             self._width = int(width)
@@ -159,17 +154,6 @@ class _PacObj(object):
         """Vector of amplitudes of shape (n_amp,) use for plotting."""
         return self._yvec
 
-    # ----------- FILT -----------
-    @property
-    def filt(self):
-        """Get the filt value."""
-        return self._filt
-
-    @filt.setter
-    def filt(self, value):
-        """Set filt value."""
-        self._speccheck(filt=value)
-
     # ----------- DCOMPLEX -----------
     @property
     def dcomplex(self):
@@ -192,17 +176,6 @@ class _PacObj(object):
         """Set cycle value."""
         self._speccheck(cycle=value)
 
-    # ----------- FILTORDER -----------
-    @property
-    def filtorder(self):
-        """Get the filtorder value."""
-        return self._filtorder
-
-    @filtorder.setter
-    def filtorder(self, value):
-        """Set filtorder value."""
-        self._speccheck(filtorder=value)
-
     # ----------- WIDTH -----------
     @property
     def width(self):
@@ -268,17 +241,11 @@ class Pac(_PacObj, _PacPlt):
     dcomplex : {'wavelet', 'hilbert'}
         Method for the complex definition. Use either 'hilbert' or
         'wavelet'.
-    filt : {'fir1', 'butter', 'bessel'}
-        Filtering method (only if dcomplex is 'hilbert'). Choose either
-        'fir1', 'butter' or 'bessel'
     cycle : tuple | (3, 6)
         Control the number of cycles for filtering (only if dcomplex is
         'hilbert'). Should be a tuple of integers where the first one
         refers to the number of cycles for the phase and the second for the
         amplitude [#f5]_.
-    filtorder : int | 3
-        Filter order for the Butterworth and Bessel filters (only if
-        dcomplex is 'hilbert').
     width : int | 7
         Width of the Morlet's wavelet.
     n_bins : int | 18
@@ -301,14 +268,13 @@ class Pac(_PacObj, _PacPlt):
     """
 
     def __init__(self, idpac=(1, 2, 3), f_pha=[2, 4], f_amp=[60, 200],
-                 dcomplex='hilbert', filt='fir1', cycle=(3, 6), filtorder=3,
-                 width=7, n_bins=18, verbose=None):
+                 dcomplex='hilbert', cycle=(3, 6), width=7, n_bins=18,
+                 verbose=None):
         """Check and initialize."""
         set_log_level(verbose)
         self._idcheck(idpac)
         _PacObj.__init__(self, f_pha=f_pha, f_amp=f_amp, dcomplex=dcomplex,
-                         filt=filt, cycle=cycle, filtorder=filtorder,
-                         width=width)
+                         cycle=cycle, width=width)
         _PacPlt.__init__(self)
         self.n_bins = int(n_bins)
         logger.info("Phase Amplitude Coupling object defined")
@@ -368,7 +334,7 @@ class Pac(_PacObj, _PacPlt):
         # ---------------------------------------------------------------------
         # copnorm if gaussian copula is used
         if self._idpac[0] == 6:
-            logger.info(f"    copnorm the phase and the amplitude")
+            logger.debug(f"    copnorm the phase and the amplitude")
             pha = copnorm(np.stack([np.sin(pha), np.cos(pha)], axis=-2))
             amp = copnorm(amp[..., np.newaxis, :])
 
@@ -402,7 +368,7 @@ class Pac(_PacObj, _PacPlt):
         return pac
 
     def filterfit(self, sf, x_pha, x_amp=None, n_perm=200, p=.05, n_jobs=-1,
-                  verbose=None):
+                  edges=None, verbose=None):
         """Filt the data then compute PAC on it.
 
         Parameters
@@ -419,6 +385,8 @@ class Pac(_PacObj, _PacPlt):
             Number of surrogates to compute.
         p : float | 0.05
             Statistical threshold
+        edges : int | None
+            Number of samples to discard to avoid edge effects due to filtering
         n_jobs : int | -1
             Number of jobs to compute PAC in parallel. For very large data,
             set this parameter to 1 in order to prevent large memory usage.
@@ -447,8 +415,9 @@ class Pac(_PacObj, _PacPlt):
         # Extract phase (npha, ...) and amplitude (namp, ...) :
         logger.info(f"    Extract phases (n_pha={len(self.xvec)}) and "
                     f"amplitudes (n_amps={len(self.yvec)})")
-        pha = self.filter(sf, x_pha, 'phase', False, n_jobs)
-        amp = self.filter(sf, x_amp, 'amplitude', False, n_jobs)
+        kw = dict(keepfilt=False, edges=edges, n_jobs=1)
+        pha = self.filter(sf, x_pha, 'phase', **kw)
+        amp = self.filter(sf, x_amp, 'amplitude', **kw)
 
         # Special cases :
         if self._idpac[0] == 5:
@@ -474,7 +443,7 @@ class Pac(_PacObj, _PacPlt):
         # ---------------------------------------------------------------------
         # check that pac and surrogates has already been computed
         assert hasattr(self, 'pac'), ("You should compute PAC first. Use the "
-                                       "`fit` method")
+                                      "`fit` method")
         assert hasattr(self, 'surrogates'), "No surrogates computed"
         assert all([isinstance(k, np.ndarray) for k in (
             self.pac, self.surrogates)])
@@ -536,7 +505,7 @@ class Pac(_PacObj, _PacPlt):
 
 
 class EventRelatedPac(_PacObj, _PacVisual):
-    """Event Related Phase-Amplitude Coupling (ERPAC).
+    """Compute the Event Related Phase-Amplitude Coupling (ERPAC).
 
     The traditional PAC approach is computed across time, hence this means that
     you can't observe PAC changes across time. In contrast, the ERPAC is
@@ -556,29 +525,21 @@ class EventRelatedPac(_PacObj, _PacVisual):
     dcomplex : {'wavelet', 'hilbert'}
         Method for the complex definition. Use either 'hilbert' or
         'wavelet'.
-    filt : {'fir1', 'butter', 'bessel'}
-        Filtering method (only if dcomplex is 'hilbert'). Choose either
-        'fir1', 'butter' or 'bessel'
     cycle : tuple | (3, 6)
         Control the number of cycles for filtering (only if dcomplex is
         'hilbert'). Should be a tuple of integers where the first one
         refers to the number of cycles for the phase and the second for the
         amplitude.
-    filtorder : int | 3
-        Filter order for the Butterworth and Bessel filters (only if
-        dcomplex is 'hilbert').
     width : int | 7
         Width of the Morlet's wavelet.
     """
 
     def __init__(self, f_pha=[2, 4], f_amp=[60, 200], dcomplex='hilbert',
-                 filt='fir1', cycle=(3, 6), filtorder=3, width=7,
-                 verbose=None):
+                 cycle=(3, 6), width=7, verbose=None):
         """Check and initialize."""
         set_log_level(verbose)
         _PacObj.__init__(self, f_pha=f_pha, f_amp=f_amp, dcomplex=dcomplex,
-                         filt=filt, cycle=cycle, filtorder=filtorder,
-                         width=width)
+                         cycle=cycle, width=width)
         _PacPlt.__init__(self)
         logger.info("Event Related PAC object defined")
 
@@ -624,6 +585,8 @@ class EventRelatedPac(_PacObj, _PacVisual):
         pha, amp = self._phampcheck(pha, amp)
         self.method = method
         self._pvalues = None
+        # move the trial axis to the end (n_freqs, n_times, n_epochs)
+        pha, amp = np.moveaxis(pha, 1, -1), np.moveaxis(amp, 1, -1)
         # method switch
         if method == 'circular':
             self.method = "ERPAC (Voytek et al. 2013)"
@@ -632,16 +595,20 @@ class EventRelatedPac(_PacObj, _PacVisual):
         elif method == 'gc':
             self.method = "Gaussian-Copula ERPAC (Ince et al. 2017)"
             logger.info(f"    Compute {self.method}")
-            self._erpac = ergcpac(pha, amp, smooth=smooth, n_jobs=n_jobs)
+            # copnorm phases and amplitudes then compute erpac
+            sco = copnorm(np.stack([np.sin(pha), np.cos(pha)], axis=-2))
+            amp = copnorm(amp)[..., np.newaxis, :]
+            self._erpac = ergcpac(sco, amp, smooth=smooth, n_jobs=n_jobs)
+            # compute permutations (if needed)
             if isinstance(n_perm, int) and (n_perm > 0):
                 logger.info(f"    Compute {n_perm} permutations")
-                self._surrogates = _ergcpac_perm(pha, amp, smooth=smooth,
+                self._surrogates = _ergcpac_perm(sco, amp, smooth=smooth,
                                                  n_jobs=n_jobs, n_perm=n_perm)
                 self.infer_pvalues(p=p)
         return self.erpac
 
     def filterfit(self, sf, x_pha, x_amp=None, method='circular', smooth=None,
-                  n_perm=None, p=.05, n_jobs=-1, verbose=None):
+                  n_perm=None, p=.05, edges=None, n_jobs=-1, verbose=None):
         """Extract phases, amplitudes and compute ERPAC.
 
         Parameters
@@ -667,6 +634,8 @@ class EventRelatedPac(_PacObj, _PacVisual):
             swapping phase trials
         p : float | 0.05
             Statistical threshold for the gaussian-copula ('gc') method
+        edges : int | None
+            Number of samples to discard to avoid edge effects due to filtering
 
         Returns
         -------
@@ -684,8 +653,9 @@ class EventRelatedPac(_PacObj, _PacVisual):
         # extract phases and amplitudes
         logger.info(f"    Extract phases (n_pha={len(self.xvec)}) and "
                     f"amplitudes (n_amps={len(self.yvec)})")
-        pha = self.filter(sf, x_pha, ftype='phase')
-        amp = self.filter(sf, x_amp, ftype='amplitude')
+        kw = dict(keepfilt=False, edges=edges, n_jobs=1)
+        pha = self.filter(sf, x_pha, ftype='phase', **kw)
+        amp = self.filter(sf, x_amp, ftype='amplitude', **kw)
         # compute erpac
         return self.fit(pha, amp, method=method, smooth=smooth, n_jobs=n_jobs,
                         n_perm=n_perm, p=p, verbose=verbose)
@@ -736,7 +706,7 @@ class EventRelatedPac(_PacObj, _PacVisual):
 
 
 class PreferredPhase(_PacObj, _PolarPlt):
-    """Evaluate the preferred phase (PP).
+    """Compute the Preferred Phase (PP).
 
     The preferred phase is defined as the phase at which the amplitude is
     maximum.
@@ -755,29 +725,21 @@ class PreferredPhase(_PacObj, _PolarPlt):
     dcomplex : {'wavelet', 'hilbert'}
         Method for the complex definition. Use either 'hilbert' or
         'wavelet'.
-    filt : {'fir1', 'butter', 'bessel'}
-        Filtering method (only if dcomplex is 'hilbert'). Choose either
-        'fir1', 'butter' or 'bessel'
     cycle : tuple | (3, 6)
         Control the number of cycles for filtering (only if dcomplex is
         'hilbert'). Should be a tuple of integers where the first one
         refers to the number of cycles for the phase and the second for the
         amplitude.
-    filtorder : int | 3
-        Filter order for the Butterworth and Bessel filters (only if
-        dcomplex is 'hilbert').
     width : int | 7
         Width of the Morlet's wavelet.
     """
 
     def __init__(self, f_pha=[2, 4], f_amp=[60, 200], dcomplex='hilbert',
-                 filt='fir1', cycle=(3, 6), filtorder=3, width=7,
-                 verbose=None):
+                 cycle=(3, 6), width=7, verbose=None):
         """Check and initialize."""
         set_log_level(verbose)
         _PacObj.__init__(self, f_pha=f_pha, f_amp=f_amp, dcomplex=dcomplex,
-                         filt=filt, cycle=cycle, filtorder=filtorder,
-                         width=width)
+                         cycle=cycle, width=width)
         _PacPlt.__init__(self)
         logger.info("Preferred phase object defined")
         self.method = 'Preferred-Phase (PP)'
@@ -810,7 +772,8 @@ class PreferredPhase(_PacObj, _PolarPlt):
         pha, amp = self._phampcheck(pha, amp)
         return preferred_phase(pha, amp, n_bins=n_bins)
 
-    def filterfit(self, sf, x_pha, x_amp=None, n_bins=12, verbose=None):
+    def filterfit(self, sf, x_pha, x_amp=None, edges=None, n_bins=12,
+                  verbose=None):
         """Extract phases, amplitudes and compute the preferred phase (PP).
 
         Parameters
@@ -827,6 +790,8 @@ class PreferredPhase(_PacObj, _PolarPlt):
         n_bins : int | 72
             Number of bins for bining the amplitude according to phase
             slices.
+        edges : int | None
+            Number of samples to discard to avoid edge effects due to filtering
 
         Returns
         -------
@@ -843,7 +808,8 @@ class PreferredPhase(_PacObj, _PolarPlt):
         # extract phases and amplitudes
         logger.info(f"    Extract phases (n_pha={len(self.xvec)}) and "
                     f"amplitudes (n_amps={len(self.yvec)})")
-        pha = self.filter(sf, x_pha, ftype='phase')
-        amp = self.filter(sf, x_amp, ftype='amplitude')
+        kw = dict(keepfilt=False, edges=edges, n_jobs=1)
+        pha = self.filter(sf, x_pha, ftype='phase', **kw)
+        amp = self.filter(sf, x_amp, ftype='amplitude', **kw)
         # compute pp
         return self.fit(pha, amp, n_bins=n_bins)
